#! /bin/sh

# Has to be EXACTLY like this, no indentation and no tabs! Otherwise
# string comparison will fail
TB_NO_SUCH_KEY="<dictionary> {
  TunnelBearNoSuchKey : true
}"


# @param String message - The message to log
readonly LOG_MESSAGE_COMMAND=$(basename "${0}")
logMessage()
{
echo "$(date '+%a %b %e %T %Y') $LOG_MESSAGE_COMMAND: "${@} >> "${SCRIPT_LOG_FILE}"
}


save_dns()
{
  logMessage "Saving current config"
  fetch_current_dns_config

	scutil <<- EOF
		open

		d.init
		d.add ServiceID ${PSID}
		set State:/Network/TunnelBear

		d.init
		d.add TunnelBearNoSuchKey true
		get State:/Network/Service/${PSID}/DNS
		set State:/Network/TunnelBear/OldDNS
		
		d.init
		d.add TunnelBearNoSuchKey true
		get Setup:/Network/Service/${PSID}/DNS
		set State:/Network/TunnelBear/OldDNSSetup

		d.init
		d.add TunnelBearNoSuchKey true
		get State:/Network/Global/DNS
		set State:/Network/TunnelBear/OldGlobalDNS
	
		d.init
		d.add TunnelBearNoSuchKey true
		get Setup:/Network/Global/DNS
		set State:/Network/TunnelBear/OldGlobalDNSSetup

		quit
	EOF
}

restore_dns()
{
  logMessage "Restoring saved config"
  fetch_saved_dns_config

  SavedPSID="$(/usr/sbin/scutil <<-EOF
    open
    show State:/Network/TunnelBear
    quit
  EOF)"

  SavedPSID="$(echo "${SavedPSID}" | grep -i '^[[:space:]]*ServiceID :' | sed -e 's/^.*: //g')"

  logMessage "SavedPSID: " ${SavedPSID}

	if [ "${DNS_OLD}" = "${TB_NO_SUCH_KEY}" ] ||
     [ "${DNS_OLD}" = "No such key" ] ; then
    logMessage "Removing DNS State"
		scutil <<- EOF
			open
			remove State:/Network/Service/${SavedPSID}/DNS
			quit
	EOF
	else
  logMessage "Restoring DNS State"
		scutil <<- EOF
			open
			get State:/Network/TunnelBear/OldDNS
			set State:/Network/Service/${SavedPSID}/DNS
			quit
	EOF
	fi

	if [ "${DNS_OLD_SETUP}" = "${TB_NO_SUCH_KEY}" ] ||
     [ "${DNS_OLD_SETUP}" = "No such key" ] ; then
    logMessage "Removing DNS Setup"
		scutil <<- EOF
			open
			remove Setup:/Network/Service/${SavedPSID}/DNS
			quit
		EOF
	else
		logMessage "Restoring DNS Setup"
		scutil <<- EOF
			open
			get State:/Network/TunnelBear/OldDNSSetup
			set Setup:/Network/Service/${SavedPSID}/DNS
			quit
		EOF
	fi

	if [ "${DNS_OLD_GLOBAL}" = "${TB_NO_SUCH_KEY}" ] ||
     [ "${DNS_OLD_GLOBAL}" = "No such key" ] ; then
    logMessage "Removing Global DNS State"
		scutil <<- EOF
			open
			remove State:/Network/Global/DNS
			quit
	EOF
	else
    logMessage "Restoring Global DNS State"
		scutil <<- EOF
			open
			get State:/Network/TunnelBear/OldGlobalDNS
			set State:/Network/Global/DNS
			quit
	EOF
	fi

	if [ "${DNS_OLD_GLOBAL_SETUP}" = "${TB_NO_SUCH_KEY}" ] ||
     [ "${DNS_OLD_GLOBAL_SETUP}" = "No such key" ] ; then
    logMessage "Removing Global DNS Setup"
		scutil <<- EOF
			open
			remove Setup:/Network/Global/DNS
			quit
		EOF
	else
		logMessage "Restoring Global DNS Setup"
		scutil <<- EOF
			open
			get State:/Network/TunnelBear/OldGlobalDNSSetup
			set Setup:/Network/Global/DNS
			quit
		EOF
	fi

  logMessage "Config restored"
  fetch_current_dns_config
}

fix_dns()
{
	# networksetup clears Setup:/ DNS settings,
	# fix them up by copying from State **only when connected***
  # using this elsewhere can causes serious DNS issues

  logMessage "Running DNS fix..."
	scutil <<- EOF
		open

		d.init
		get State:/Network/Service/${PSID}/DNS
		set Setup:/Network/Service/${PSID}/DNS

		d.init
		get State:/Network/Global/DNS
		set Setup:/Network/Global/DNS

		quit
	EOF
}

cleanup()
{
  logMessage "Cleaning up..."
	scutil <<- EOF
		open
		remove State:/Network/TunnelBear/OldDNS
		remove State:/Network/TunnelBear/OldDNSSetup
		remove State:/Network/TunnelBear/OldGlobalDNS
		remove State:/Network/TunnelBear/OldGlobalDNSSetup
		remove State:/Network/TunnelBear
		quit
	EOF
}

fetch_current_dns_config()
{
	DNS="$(/usr/sbin/scutil <<-EOF
		open
		show State:/Network/Service/${PSID}/DNS
		quit
	EOF)"

	DNS_SETUP="$(/usr/sbin/scutil <<-EOF
		open
		show Setup:/Network/Service/${PSID}/DNS
		quit
	EOF)"

	DNS_GLOBAL="$(/usr/sbin/scutil <<-EOF
		open
		show State:/Network/Global/DNS
		quit
	EOF)"

	DNS_GLOBAL_SETUP="$(/usr/sbin/scutil <<-EOF
		open
		show Setup:/Network/Global/DNS
		quit
	EOF)"

	logMessage "DNS: " ${DNS}
	logMessage "DNS_SETUP: " ${DNS_SETUP}
	logMessage "DNS_GLOBAL: " ${DNS_GLOBAL}
	logMessage "DNS_GLOBAL_SETUP: " ${DNS_GLOBAL_SETUP}
  logMessage "PSID: " ${PSID}
}

fetch_saved_dns_config()
{
	DNS_OLD="$(/usr/sbin/scutil <<-EOF
		open
		show State:/Network/TunnelBear/OldDNS
		quit
	EOF)"

	DNS_OLD_SETUP="$(/usr/sbin/scutil <<-EOF
		open
		show State:/Network/TunnelBear/OldDNSSetup
		quit
	EOF)"

	DNS_OLD_GLOBAL="$(/usr/sbin/scutil <<-EOF
		open
		show State:/Network/TunnelBear/OldGlobalDNS
		quit
	EOF)"

	DNS_OLD_GLOBAL_SETUP="$(/usr/sbin/scutil <<-EOF
		open
		show State:/Network/TunnelBear/OldGlobalDNSSetup
		quit
	EOF)"

	logMessage "DNS_OLD: " ${DNS_OLD}
	logMessage "DNS_OLD_SETUP: " ${DNS_OLD_SETUP}
	logMessage "DNS_OLD_GLOBAL: " ${DNS_OLD_GLOBAL}
	logMessage "DNS_OLD_GLOBAL_SETUP: " ${DNS_OLD_GLOBAL_SETUP}
}

################# START HERE ####################

readonly SCRIPT_LOG_FILE="/var/log/tbeard/dnsManager.log"

OPERATION="$1"

PSID="$(
	scutil <<-EOF |
		open
		show State:/Network/Global/IPv4
		quit
	EOF
	grep PrimaryService | sed -e 's/.*PrimaryService : //'
)"

if [ "$OPERATION" == "save" ]; then
  save_dns
elif [ "$OPERATION" == "restore" ]; then
  restore_dns
  cleanup
elif [ "$OPERATION" == "fix" ]; then
  fix_dns
fi
